//
// Copyright (c) 2016 comScore. All rights reserved.
//

#import <Foundation/Foundation.h>

/**
  The SCOREventInfo object allows to send different information for different clients in 
  the same app.
 
  @warning This class is not thread-safe.
 
  EventInfo must be used when the event that needs to be send requies some customization
  or to include or not other publishers.
 
  The following code shows how to add some specific labels for 2 publishers:
 
     @code
     SCOREventInfo *eventInfo = [[SCOREventInfo alloc] initWithPublishers:@[@"1000001",@"1000002"]];
     // Specify an extra label for the publisher ID "1000001"
     [eventInfo setPersistentLabelWithPublisherId:@"1000001" name:@"LabelA" value:@"valueA"];
     // Specify an extra label for the publisher ID "1000002"
     [eventInfo setPersistentLabelWithPublisherId:@"1000002" name:@"LabelB" value:@"valueB"];
     // Some labels for all the clients that we specified
     [eventInfo setPersistentLabelWithName:@"LabelC" value:@"valueC"];

 
     // The view event will be send with some extra labels for the publishers "1000001" and "1000002"
     [SCORAnalytics notifyViewEventWithEventInfo:eventInfo];
     @endcode
 
 */
@interface SCOREventInfo : NSObject
@property(readonly) NSDictionary *persistentLabels;
@property(readonly) NSDictionary *publisherSpecificPersistentLabels;
@property(readonly) NSArray *publishers;

/**
 *  Initialize a SCOREventInfo configured for the given publishers.
 *
 *  @param publishers NSArray with publishers Ids
 *
 *  @return SCOREventInfo or subclass
 */
- (instancetype)initWithPublishers:(NSArray *)publishers;

/**
 *  Removes all custom persistent labels that are not exclusive to any specific publisher.
 */
- (void)removeAllPersistentLabels;

/**
 *  Sets a new gloabl label.
 *
 *  @param name  NSString, label name
 *  @param value NSString, label value
 */
- (void)setPersistentLabelWithName:(NSString *)name value:(NSString *)value;

/**
 *  Adds or update the given gloabl labels. This will replace any previous label that 
 *  is also included in the provided dictionary.
 *
 *  @param persistentLabels NSDictionary of strings.
 */
- (void)setPersistentLabels:(NSDictionary *)persistentLabels;

/**
 *  Retrieves the value of the given persistent label. This method will not retrieve 
 *  labels exclusive for a specific publisher, use <persistentLabelWithPublisherId:name:>
 *  instead. If you need to retrieve several labels direclty use <persistentLabels>.
 *
 *  @param name NSString, label name
 *
 *  @return NSString or *nil*
 */
- (NSString *)persistentLabelWithName:(NSString *)name;

/**
 *  Removes the persistent label whith the given name, if any.
 *
 *  @param name NSString, label name
 */
- (void)removePersistentLabelWithName:(NSString *)name;

/**
 *  Adds or update the given labels exclusively to the provided publisherId. This will replace 
 *  any previous label that is also included in the provided dictionary.
 *
 *  @param publisherId NSString, publiher id
 *  @param labels      NSDictionary, labels dictionary
 */
- (void)setPersistentLabelsWithPublisherId:(NSString *)publisherId labels:(NSDictionary *)labels;

/**
 *  Returns all the persistent labels thet are exclusive to the given publisher Id.
 *
 *  @param publisherId NSString, publisher id
 *
 *  @return NSDictionary or *nil*
 */
- (NSDictionary *)persistentLabelsWithPublisherId:(NSString *)publisherId;

/**
 *  Removes all persistent labels associated with the given.
 *
 *  @param publisherId NSString, publisher id
 */
- (void)removeAllPersistentLabelsWithPublisherId:(NSString *)publisherId;

/**
 *  Sets the given label exclusively to the provided publisher id.
 *
 *  @param publisherId NSString, publisher id
 *  @param name        NSString, label name
 *  @param value       NSString, label value
 */
- (void)setPersistentLabelWithPublisherId:(NSString *)publisherId name:(NSString *)name value:(NSString *)value;

/**
 *  Returns the value of a persistent label exclusive to the provided publisher id.
 *
 *  @param publisherId NSString, publisher id
 *  @param name        NSSrtring, label name
 *
 *  @return NSString or *nil*
 */
- (NSString *)persistentLabelWithPublisherId:(NSString *)publisherId name:(NSString *)name;

/**
 *  Removes the given label that was exclusive to the provided publisher id.
 *
 *  @param publisherId NSString, publisher id
 *  @param name        NSSrtring, label name
 */
- (void)removePersistentLabelWithPublisherId:(NSString *)publisherId name:(NSString *)name;
@end