/**
* Media Acceleration SDK
*
* Copyright (c) 2016 Akamai Technologies, Inc. All Rights Reserved.
*
* The following software code is provided to you under license from Akamai Technologies, Inc.
* The applicable license was supplied with the download of this code and is
* otherwise available upon request from Akamai Technologies, Inc.
* Any use of this software code constitutes acceptance of the applicable license.
*/

/*!
 * @header MediaAcceleration
 * @abstract The Media Acceleration SDK for iOS and tvOS
 */

#import <Foundation/Foundation.h>

/*!
 * A certificate chain verification request for a given hostname and scheme.
 */
@interface CertificateChainVerificationRequest : NSObject

/*!
 * The certificate chain starting with the end-user certificate. Each element in the chain
 * is the binary DER encoding of a X.509v3 certificate. The chain is guaranteed to be non-empty.
 */
@property (readonly) unsigned char *_Nonnull *_Nonnull certificateChain;

/*!
 * The scheme which will be either "http" or "https". This corresponds to the scheme in the URI given
 * to one of the URI conversion methods, e.g. `getLocalhostURI`. Note that this will always be "https"
 * unless the SDK is communicating over the QUIC protocol in which case it may also be "http".
 */
@property (readonly, nonnull) NSString* scheme;

/*!
 * The hostname to verify the certificate chain for.
 */
@property (readonly, nonnull) NSString* hostname;

/*!
 * Indicates if the hostname returned by `getHostname` matches the end-user certificate
 * in the value returned by `getCertificateChain`.
 */
@property (readonly) BOOL hostnameMatchesCertificate;

@end

/*!
 * @abstract A custom certificate chain verifier to verify `CertificateChainVerificationRequest`.
 *
 * @discussion Applications that want to perform custom certificate chain verification using
 * `setCustomCertificateVerifier`, must implement this interface and the
 * `isCertificateChainTrusted` method must be overridden.
 */
@interface CertificateChainVerifier : NSObject

/*!
 * @abstract This method verifies a `CertificateChainVerificationRequest`.
 *
 * @discussion This method will always be called in one of the global concurrent dispatch queues.
 *
 * @param info The certificate chain verification request to be verified.
 * @param queue The target queue on which the completion handler should be run.
 * @param callback The completion handler must be called with the result of the verification (YES in case the certificate chain and hostname are accepted, NO otherwise), and in case of NO, a human-readable and informative error message should be set.
 */
-(void)isCertificateChainTrusted:(nonnull CertificateChainVerificationRequest *)info targetQueue:(nonnull dispatch_queue_t)queue completionHandler:(void(^ _Nonnull)(BOOL isVerified, NSString* _Nullable errorMessage))callback;

@end

/*!
 * @typedef NS_ENUM
 * @brief Enumeration for log constants.
 */
typedef NS_ENUM(NSInteger, MediaAccelerationLogLevel) {
    /*!
     * Log severity for information about code execution. This level may have a significant negative impact on performance.
     */
    MediaAccelerationLogLevelVerbose,
    
    /*!
     * Log severity for information about code execution. Setting this level may have a partial negative impact on performance.
     */
    MediaAccelerationLogLevelDebug,
    
    /*!
     * Log severity for information about normal operation.
     */
    MediaAccelerationLogLevelInfo,
    
    /*!
     * Log severity for notifications about unexpected states without a significant impact.
     */
    MediaAccelerationLogLevelWarning,
    
    /*!
     * Log severity for notifications about abnormal states with a significant impact. This is the default log level used
     * by the SDK.
     */
    MediaAccelerationLogLevelError,
    
    /*!
     * Log severity meaning that logging is turned off.
     */
    MediaAccelerationLogLevelOff
};

/*!
 * Local option for sprotocol.
 *
 * This can be used to disable QUIC.
 *
 * Values: `LOCALOPT_VAL_TCP`
 */
static const NSString* _Nonnull LOCALOPT_KEY_SPROTOCOL = @"sprotocol";

/*!
 * Setting `SPROTOCOL` to this will disable QUIC.
 */
static const NSString* _Nonnull LOCALOPT_VAL_TCP = @"tcp";

/*!
 * This class contains the API of the Akamai Media Acceleration SDK.
 *
 * This API is non-blocking, thread-safe and reentrant. All calls to methods with
 * completion handlers in the API are serialized, and calls to methods without completion
 * handlers are not serialized.
 */
@interface MediaAcceleration : NSObject

/* ------------------------------------------------------
 *  Starting and stopping the Media Acceleration SDK
 * ------------------------------------------------------
 */

/*!
 * @functiongroup Starting and stopping the Media Acceleration SDK
 */

/*!
 * @abstract Starts the SDK. It will allocate some memory and create several threads for internal use.
 *
 * @discussion When the completion handler is called with YES, the SDK will have started with its default settings.
 * This includes:
 * a) The SDK will use the system default method of verifying certificate chains, and
 * b) All calls, including previous information given to `allowCookieSharing`, will be forgotten.
 *
 * To enable AirPlay support, the SDK will register a Bonjour service browseable on the `local.` domain. 
 * The SDK will use the Bonjour hostname of the device for the `getAirPlayURI` converted links to be accessible on the LAN.
 *
 * @note This method can also be used on tvOS, but please note that tvOS does not support streaming to other devices via AirPlay.
 * @param logLevel Configures the minimum log level the SDK will log messages to the Apple System Log. Note that enabling logging may reveal information about the user behavior to the Apple System Log, including the visited URLs. See `setLogLevel` for more details.
 * @param queue The target queue on which the completion handler should be run.
 * @param enableAirPlay If YES, the SDK is started with AirPlay support. 
 * @param callback The completion handler to be called on the given target queue with a value indicating if the SDK was started by this call.
 *
 * @since 1.0
 */
+(void)startWithLogLevel:(MediaAccelerationLogLevel)logLevel targetQueue:(dispatch_queue_t _Nonnull)queue enableAirPlay:(BOOL)enableAirPlay completionHandler:(void (^ _Nonnull)(BOOL success))callback;

/*!
 * @abstract This is equivalent to calling `startWithLogLevel` with the default `MediaAccelerationLogLevelError` log level and `enableAirPlay` set to YES. Note that enabling logging may reveal information about the user behavior to the Apple System Log, including which URLs are visited.
 * @since 1.0
 */
+(void)startWithTargetQueue:(dispatch_queue_t _Nonnull)queue completionHandler:(void (^ _Nonnull)(BOOL))callback;


/*!
 * @abstract Stops the SDK.
 *
 * @discussion When the method has returned, most of the threads and most of the memory allocated by the SDK will have been released.
 *
 * After this method has returned, a call to one of the `start` methods will start the SDK again.
 *
 * Multiple calls to `start` and `stop` should not create any memory leaks or other leaks.
 *
 * Multiple calls without calling `start` in between have no effect.
 *
 * @param queue The target queue on which the completion handler should be run. If `NULL`, the SDK does not run the completionHandler.
 * @param callback The completion handler to be called once the SDK has stopped. If nil, no completion handler is executed. If not nil, when this completion handler is called, all other callbacks made by the SDK have finished. 
 *
 * @since 1.0
 */
+(void)stopWithTargetQueue:(dispatch_queue_t _Nullable)queue completionHandler:(void (^ _Nullable)(void))callback;

/* -------------------------------------------------------------------
 * Converting URIs to URIs handled by the Media Acceleration SDK
 * -------------------------------------------------------------------
 */

/*!
 * @functiongroup Converting URIs to URIs handled by the Media Acceleration SDK
 */

/*!
 * @abstract Callback used when converting a user-specified URI to a URI handled by the Media Acceleration SDK.
 *
 * @param convertedURI nil if the conversion failed, and otherwise a URI pointing to an HTTP server bound by the SDK.
 * @param isAirPlayURI YES if the `convertedURI` is accessible by devices on the LAN, NO otherwise.
 * @param errorCode If `convertedURI` is nil, this will be set to an error code representing the error that occurred.
 * @param errorMsg if `convertedURI` is nil, this will be set to a human-readable error message representing the error that occurred.
 *
 * @since 1.0
 */
typedef void (^URICallback)(NSURL* _Nullable convertedURI, BOOL isAirPlayURI, int errorCode, NSString* _Nullable errorMsg);

/*!
 * @abstract Converts the provided absolute URI to a URI pointing to an HTTP server bound by the SDK on localhost.
 *
 * @discussion The local options map can be used to modify the behaviour of the SDK from its defaults. It is a
 * map from string constants of this class starting with `LOCALOPT_KEY_` to string constants starting with `LOCALOPT_VAL_`.
 * Unknown or invalid options will be silently ignored. Setting this argument to nil is equivalent to setting it to an empty map.
 *
 * @param uri The URI to convert. This must have a format as described in RFC 3986 and only the "http" and "https" schemes are supported.
 * @param localOptions Options to put into the returned URI on which the SDK will react to when processing requests for the returned URI.
 * @param queue The target queue on which the completion handler should be run.
 * @param callback The completion handler to be called once the converted URI is ready. In case of errors the converted URI will be set to nil.
 *
 * @since 1.0
 */
+(void)getLocalhostURI:(nonnull NSURL*)uri localOptionsMap:(nullable NSDictionary<NSString*, NSObject*>*)localOptions targetQueue:(dispatch_queue_t _Nonnull)queue completionHandler:(URICallback _Nonnull)callback;

/*!
 * @abstract Converts the provided absolute URI to a URI pointing to an HTTP server bound by the SDK, potentially accessible by devices on the LAN.
 *
 * @discussion The local options map can be used to modify the behaviour of the SDK from its defaults. It is a
 * map from string constants of this class starting with `LOCALOPT_KEY_` to string constants starting with `LOCALOPT_VAL_`.
 * Unknown or invalid options will be silently ignored. Setting this argument to nil is equivalent to setting it to an empty map.
 *
 * If the MediaAcceleration SDK was started with `enableAirPlay:NO`, or if the SDK is unable to register the Bonjour service, \
 * this method will return the same URI as the one returned by `getLocalhostURI`, and the callback's `isAirPlayURI` argument will be NO. 
 *
 * @param uri The URI to convert. This must have a format as described in RFC 3986 and only the "http" and "https" schemes are supported.
 * @param localOptions Options to put into the returned URI on which the SDK will react to when processing requests for the returned URI.
 * @param queue The target queue on which the completion handler should be run.
 * @param callback The completion handler to be called once the converted URI is ready. In case of errors the converted URI will be set to nil.
 *
 * @since 1.0
 */
+(void)getAirPlayURI:(nonnull NSURL*)uri localOptionsMap:(nullable NSDictionary<NSString*, NSObject*>*)localOptions targetQueue:(dispatch_queue_t _Nonnull)queue completionHandler:(URICallback _Nonnull)callback;

/*-------------------------------------------------------------------
 * Certificate validation
 * -------------------------------------------------------------------
 */

/*!
 * @functiongroup Certificate validation
 */

/*!
 * @abstract Configures the SDK to verify X.509 certificates for a given hostname using a
 * subclass of `CertificateChainVerifier` with an overriden `isCertificateChain`
 * method. This is intended to be used only in cases where the system default
 * way of verifying certificate chains is not desired.
 *
 * @discussion When called with nil, the certificate verifier will be unset, and
 * the system default method of certificate chain verification will be used.
 *
 * @param verifier A subclass of `CertificateChainVerifier` to be used for verifying X.509 certificate chains.
 *
 * @since 1.0
 */
+(void)setCustomCertificateVerifier:(nullable CertificateChainVerifier*)verifier;

/* -------------------------------------------------------------------
 * Synchronous, non-blocking
 * -------------------------------------------------------------------
 */

/*!
 * @functiongroup Runtime configuration options
 */

/*!
 * @abstract Adds a domain under which cookies can be shared.
 *
 * @discussion Calling this only affects the currently started SDK, and will be ignored if the SDK is not started.
 *
 * The empty string can be used as domain to allow cookie sharing between all domains.
 *
 * @param scheme This should be "http" or "https" indicating the scheme (in the argument to `getLocalhostURI`) to allow cookie sharing for.
 * @param domain The domain under which cookies can be shared. This should never start with a dot.
 *
 * @return YES if the call was successful; NO otherwise.
 *
 * @since 1.0
 */
+(BOOL)allowCookieSharingWithScheme:(nonnull NSString*)scheme domain:(nonnull NSString*)domain;

/*!
 * @abstract Configures the minimum level of log messages the SDK will log to the Apple System Log.
 *
 * @discussion Possible values: `MediaAccelerationLogLevel*` string constants.
 *
 * Note that enabling logging may reveal information about the user behavior to the Apple System Log facility, including which URLs are visited.
 *
 * Logging levels are ordered as follows: 
 *
 *  + `MediaAccelerationLogLevelVerbose` > 
 *  + `MediaAccelerationLogLevelDebug` > 
 *  + `MediaAccelerationLogLevelInfo` > 
 *  + `MediaAccelerationLogLevelWarning` > 
 *  + `MediaAccelerationLogLevelError` > 
 *  + `MediaAccelerationLogLevelOff`
 *
 * where `MediaAccelerationLogLevelVerbose` enables the maximal amount of logging, 
 * and `MediaAccelerationLogLevelOff` completely disables logging.
 *
 * @note Calling this only affects the currently started SDK, and will be ignored if the SDK is not started.
 * @param level The minimum log level the SDK will log to the Apple System Log.
 * @return YES if the log level was successfully set, NO otherwise.
 *
 * @since 1.0
 */
+(BOOL)setLogLevel:(MediaAccelerationLogLevel)level;

/*!
 * @abstract Retrieves the Media Acceleration SDK's internal version number.
 *
 * @discussion This method can be called at any time, and will always return the same value.
 * @return The Media Acceleration SDK's internal version number.
 *
 * @since 1.0
 */
+(NSString* _Nullable)version;

@end


// "Private" methods, i.e. not intended for public use.
@interface MediaAcceleration ()
+(BOOL)invokeWithKey:(NSString* _Nonnull) key value:(NSString* _Nonnull)value;
+(CertificateChainVerifier* _Nullable)getCustomCertificateVerifier;
@end

@interface CertificateChainVerificationRequest ()
-(id _Nullable)initWithCertificates:(unsigned char* _Nonnull * _Nonnull)certs host:(NSString* _Nonnull)host scheme:(NSString* _Nonnull)scheme hostnameMatchesCertificate:(BOOL)hostnameMatchesCertificate;
@end
